% Matlab for generating last three figures in Appendix B of
% https://ccrma.stanford.edu/~jos/huygens/

clear all; clf;

temp = 20; % C
[c,rho] = soundspeed(temp); % set default constants, soundspeed.m

f = 1000; % Hz
nCTaper = 3; % number of wavelengths over which to taper each side of the array
xMin = -6; % evaluate 6 meters to the left (listening line)
xMax = 6;  %      and 6 to the right
xMinArray = -3; % array extent
xMaxArray =  3;
lFactor = 2; % lambda/lFactor is the driver spacing on the array

w = 2*pi*f;
k = w/c; % rad/s / (m/s) = rad/m

lambda = 2*pi/k;
dx = lambda/10;
zl = lambda; % m
%zl = lambda/4; % interesting analytical case
sep = lambda/lFactor;

NOLA = floor(0.5*(xMaxArray-xMinArray)/sep); % number of overlap-adds to left AND to right

figure(1);
subplot(2,1,1); % slice in x
  x = [xMin:dx:xMax];
  Nx = length(x);
  px = 1 ./ sqrt(1 + (x ./ zl).^2);
  rd = sqrt(zl^2+ x.^2) - zl;
  phase = k*rd;
  pxp = px .* cos(phase);
  plot(x,pxp,'linewidth',3);
  grid('on');
  xlabel('Position (m)');
  ylabel('Gain');
  title(sprintf('Pt-Source Freq-Response Real Part, L Dist %0.2f m, Freq %d kHz',zl,f/1000));
subplot(2,1,2); % slice in theta
  tx = [-pi/2:dx/100:pi/2];
  Ntx = length(tx);
  ptx = 1 ./ sqrt(1 + tan(tx).^2);
  rd = zl*(sqrt(1+tan(tx).^2) - 1);
  phaset = k*rd;
  ptxp = ptx .* cos(phaset);
  plot(tx*180/pi,ptxp,'linewidth',3);
  grid('on');
  xlabel('Angle (degrees)');
  ylabel('Gain');
  title('Same plot versus angle to listener');
print -depsc '../eps/pointSourceFRSlices.eps'

figure(2);
  nsep = 2;
  x = [-5*lambda-(nsep-1)*sep/2:dx/10:5*lambda+(nsep-1)*sep/2];
  Nx = length(x);
  pxps = zeros(nsep,Nx); % individual complex pressure slices
  for (isep=1:nsep)
    xs = x - (nsep-1)*sep/2 + (isep-1)*sep;
    pxs = 1 ./ sqrt(1 + (xs ./ zl).^2);
    rds = sqrt(zl^2+ xs.^2) - zl;
    phases = k*rds;
    pxps(isep,:) = pxs .* exp(j*phases);
  end
  subplot(2,1,1);
  pxpsum = sum(pxps);
  plot(x,abs(pxpsum),'-k','linewidth',3);
  grid('on');
  hold('on');
  plot(x,real(pxps)',':k','linewidth',1);
  hold('off');
  xlabel('Position (m)');
  ylabel('Gain (Linear)');
  title(sprintf('Pt-Src Freq-Resp Modulus, L Dist %0.2f m, Spacing lambda/%d, Freq %d kHz',zl,lFactor,f/1000));
  subplot(2,1,2);
  plot(x,db(pxpsum,-40),'-k','linewidth',3);
  grid('on');
  hold('on');
  plot(x,db(real(pxps),-40)',':k','linewidth',1);
  hold('off');
  title('Same plots of absolute value on a dB vertical scale');
  xlabel('Position (m)');
  ylabel('Gain (dB)');
  fn = sprintf('../eps/ptSrcFRSliceLFactor%dzlf%d.eps',lFactor,zl/lambda);
  cmd = sprintf('print -depsc %s',fn);
  disp(cmd);
  eval(cmd);

for i=[0,1]
  figure(3+i);
  taper = i * nCTaper;
  xMinOLA = floor(xMin - NOLA*sep);
  xMaxOLA = ceil(xMax + NOLA*sep);
  xAll = [xMinOLA:dx:xMaxOLA];
  nXAll = length(xAll);
  olaBuffer = zeros(1,nXAll);
  w = ones(1,nXAll);
  nTaper = round(taper*lambda/sep);
  wh = hann(2*nTaper+2);
  wh = wh(2:end-1); % toss initial and trailing zero
  nHops = 2*NOLA+1;
  w = [wh(1:nTaper)',ones(1,nHops-2*nTaper),wh(nTaper+1:2*nTaper)']; % window
  spreadingLossComp = sqrt(zl) * (sep/(1.7*lambda))/zl; % sqrt(zl) = compensation part
  for (m=-NOLA:NOLA)
    xola = [xMinOLA+m*sep:dx:xMaxOLA+m*sep];
    pxola = 1 ./ sqrt(1 + (xola ./ zl).^2);
    rdola = sqrt(zl^2+ xola.^2) - zl;
    phaseola = k*rdola;
    pxpola = pxola .* exp(j*phaseola);
    olaBuffer = olaBuffer + pxpola * w(m+NOLA+1) * spreadingLossComp;
  end
  xw = [-NOLA*sep:sep:+NOLA*sep]; % Assumes array is centered on x=0
  subplot(2,1,1);
  plot(xAll,abs(olaBuffer),'linewidth',3);
  hold('on');
  plot(xw,w,'ok','linewidth',1,'markersize',5);
  hold('off');
  grid('on');
  xlabel('Position (m)');
  ylabel('Gain (Linear)');
  title(sprintf('Pt-Source Freq-Response %d-OLA Modulus, Taper %d, L Dist %0.2f m, Freq %d kHz',...
		2*NOLA+1,taper,zl,f/1000));
  subplot(2,1,2);
  plot(xAll,db(olaBuffer),'linewidth',3);
  hold('on');
  plot(xw,db(w),'ok','linewidth',1,'markersize',5);
  hold('off');
  grid('on');
  xlabel('Position (m)');
  ylabel('Gain (dB)');
  title('Same plot on a dB vertical scale');

  fn = sprintf('../eps/pointSourceFRSliceOLA%dTaper%dzlf%d.eps',2*NOLA+1,taper,zl/lambda);
  cmd = sprintf('print -depsc %s',fn);
  disp(cmd);
  eval(cmd);
end
